package com.sample.servlet;

import java.io.IOException;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.naming.InitialContext;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.UserTransaction;

import org.drools.runtime.StatefulKnowledgeSession;
import org.jbpm.process.audit.NodeInstanceLog;
import org.jbpm.task.query.TaskSummary;

import com.sample.processserver.JbpmAPIUtil;

/**
 * Servlet implementation class OrderFactoryServlet
 */
public class OrderFactoryServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;

	/**
	 * Default constructor.
	 */
	public OrderFactoryServlet() {
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		execute(request, response);

	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException,
			IOException {
		execute(request, response);
	}

	public static void execute(HttpServletRequest request, HttpServletResponse response) {
		RequestDispatcher dispatcher = null;

		try {
			printThreadDump();
			String proInstIdStr = request.getParameter("processInstanceID");
			proInstIdStr = (proInstIdStr==null) ? "0":proInstIdStr;
			String selectedTask = request.getParameter("selectedTask");
			int selectedTaskId = -1;
			if (selectedTask != null) {
				selectedTaskId = Integer.parseInt(selectedTask);
			}

			String param = "";
			String isStartedParam = "";
			if (selectedTask != null && selectedTaskId != -1) {
				String paramStatus = request.getParameter(selectedTask);
				int adjChar = paramStatus.indexOf("-");
				param = paramStatus.substring(adjChar + 1, paramStatus.length());
				isStartedParam = paramStatus.substring(0, adjChar);
			} else {
				String paramStatus = request.getParameter("taskDetails");
				if (paramStatus != null) {
					param = "taskDetails";
				}
			}

			String isAutoRepStatusParam = request.getParameter("isAutoRepStatusParam" + selectedTask);
			System.out.println("OrderFactoryServlet.execute() isAutoRepStatusParam:" + isAutoRepStatusParam);
			isAutoRepStatusParam = (isAutoRepStatusParam != null) ? isAutoRepStatusParam : "";

			String userName = request.getParameter("userName");
			userName = (userName != null) ? userName : "";

			boolean isStarted = false;
			if (isStartedParam.equalsIgnoreCase("true")) {
				isStarted = true;
			}
			boolean isAutoRepStatus = false;
			if (isAutoRepStatusParam.equalsIgnoreCase("true")) {
				isAutoRepStatus = true;
			}

			List<TaskSummary> taskResults = null;

			String isBusinessProcessStarted = "false";
			UserTransaction ut = (UserTransaction) new InitialContext().lookup("java:comp/UserTransaction");
			ut.begin();
			StatefulKnowledgeSession ksession = JbpmAPIUtil.getSession();
			TaskWorkUtil.startProcessExecutionLog(ksession);
			
			if (param.equalsIgnoreCase("startProcess")) {
				boolean isStatus = false;
				try {
					ksession.startProcess("com.sample.bpmn.orderfactory");
					
					isStatus = true;
				} catch (Exception e) {
					isStatus = false;
					e.printStackTrace();
				}
				isBusinessProcessStarted = (isStatus) ? "true" : "false";
				taskResults = TaskWorkUtil.getTaskDetails(userName);

			} else if (param.equalsIgnoreCase("taskDetails")) {
				taskResults = TaskWorkUtil.getTaskDetails(userName);

			} else if (param.equalsIgnoreCase("reviewProductListStart")) {
				taskResults = TaskWorkUtil.startCompleteReviewProductPP(isStarted, isAutoRepStatus, selectedTaskId);
				taskResults = TaskWorkUtil.getTaskDetails(userName);
			} else if (param.equalsIgnoreCase("reviewProductListComplete")) {
				taskResults = TaskWorkUtil.startCompleteReviewProductPP(isStarted, isAutoRepStatus, selectedTaskId);
				taskResults = TaskWorkUtil.getTaskDetails(userName);
			} else if (param.equalsIgnoreCase("startComplete")) {
				taskResults = TaskWorkUtil.startCompleteTask(isStarted, userName, selectedTaskId);
				taskResults = TaskWorkUtil.getTaskDetails(userName);
			}
			
			ksession.fireAllRules();
			ut.commit();
			
			long processInstanceId = Long.parseLong(proInstIdStr);

			List<NodeInstanceLog> nodeInstanceLog = TaskWorkUtil.getProcessInstaceLogger().findNodeInstances(processInstanceId);
			sortLogsItems(nodeInstanceLog);
			request.setAttribute("nodeInstanceLog", nodeInstanceLog);
			request.setAttribute("taskResults", taskResults);
			request.setAttribute("isBusinessProcessStarted", isBusinessProcessStarted);

			String pageName = "";
			if (userName.equalsIgnoreCase("Administrator")) {
				pageName = "Administrator.jsp";
			} else if (userName.equalsIgnoreCase("krisv")) {
				pageName = "ProductPlanner.jsp";
			} else if (userName.equalsIgnoreCase("john")) {
				pageName = "PNSTeam.jsp";
			} else if (userName.equalsIgnoreCase("mary")) {
				pageName = "InventoryTeam.jsp";
			}

			dispatcher = request.getRequestDispatcher("./" + pageName);
			dispatcher.include(request, response);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	

	private static void sortLogsItems(List<NodeInstanceLog> nodeInstanceLog) {
		Collections.sort(nodeInstanceLog, new Comparator(){

			public int compare(Object arg0, Object arg1) {
				NodeInstanceLog log1 = (NodeInstanceLog) arg0;
				NodeInstanceLog log2 = (NodeInstanceLog) arg1;
				return (int) (log1.getId()-log2.getId());
			}
			
		});
		
	}

	/**
	 * 
	 */
	private static void printThreadDump() {
		Map threadMap = Thread.getAllStackTraces();
		for (Iterator threadMapIT = threadMap.keySet().iterator(); threadMapIT.hasNext();) {
			Thread th = (Thread) threadMapIT.next();
			if (th.getName().contains("NioProcessor")) {
				System.out.println("\nThread Dump _____________" + th.getId() + " - " + th.getName());
			}
		}
	}

}
